@@ -7,6 +7,7 @@ from typing import Dict, List
 from sacrerouge.commands import Subcommand
 from sacrerouge.common import Params
 from sacrerouge.common.logging import prepare_global_logging
+from sacrerouge.common.util import import_module_and_submodules
 from sacrerouge.data import EvalInstance, Metrics
 from sacrerouge.data.dataset_readers import DatasetReader
 from sacrerouge.io import JsonlWriter
@@ -15,6 +16,41 @@ from sacrerouge.metrics import Metric
 logger = logging.getLogger(__name__)
 
 
+def add_score_arguments(parser: argparse.ArgumentParser, include_config_arguments: bool) -> None:
+    if include_config_arguments:
+        parser.add_argument(
+            'config',
+            type=str,
+            help='The config file that specifies the dataset reader and metrics'
+        )
+        parser.add_argument(
+            '--overrides',
+            type=str,
+            help='A serialized json that will override the parameters passed in "config"'
+        )
+
+    parser.add_argument(
+        'output_jsonl',
+        type=str,
+        help='The path to where the input-level metrics should be written'
+    )
+    parser.add_argument(
+        '--log-file',
+        type=str,
+        help='The file where the log should be written'
+    )
+    parser.add_argument(
+        '--silent',
+        action='store_true',
+        help='Controls whether the log should be written to stdout'
+    )
+    parser.add_argument(
+        '--include-packages',
+        nargs='+',
+        help='A list of additional packages to include'
+    )
+
+
 def _load_metrics(params: Params) -> List[Metric]:
     metrics = []
     for metric_params in params.pop('metrics'):
@@ -26,49 +62,78 @@ def _load_metrics(params: Params) -> List[Metric]:
 def _score_with_metric(metric: Metric,
                        instances: List[EvalInstance],
                        metrics_dicts: Dict[str, Dict[str, Metrics]]) -> None:
+    # The summaries need to be grouped based on identical context. For instance, we group all of the summaries
+    # that have the same reference documents together. This can sometimes make calculating the metric faster. The
+    # following variables assist doing this.
+    #
+    # Maintains a list of the unique contexts which group the summaries
     fields_list = []
+
+    # A mapping from the context to its index in `fields_list`
     field_to_index = {}
+
+    # A nested list that will be parallel to `fields_list`. The entry at index `i` is a list of instances which should
+    # be scored with `fields_list[i]`
     instances_list = []
+
+    # A nested list that will be parallel to `instances_list` which contains the summary-specific fields
+    # for the corresponding instance
+    summary_fields_lists = []
+
+    # A nested list that will be parallel to `instances_list` which marks if the calculation for that (summary, context)
+    # pair represents jackknifing or not
     jackknifing_flags = []
 
     for instance in instances:
         # Select just the relevant fields for this metric
-        fields = instance.fields.select_fields(metric.required_fields)
+        summary_fields = instance.fields.select_fields(metric.required_summary_fields)
+        context_fields = instance.fields.select_fields(metric.required_context_fields)
 
         # Score the instance normally using all of the fields. However,
         # if the metric requires jackknifing and this is a reference summary,
         # the metric is comparable to the jackknifing metrics.
         is_jackknifing = metric.requires_jackknifing() and instance.summarizer_type == 'reference'
 
-        if fields not in field_to_index:
-            field_to_index[fields] = len(field_to_index)
-            fields_list.append(fields)
+        if context_fields not in field_to_index:
+            field_to_index[context_fields] = len(field_to_index)
+            fields_list.append(context_fields)
             instances_list.append([])
+            summary_fields_lists.append([])
             jackknifing_flags.append([])
 
-        index = field_to_index[fields]
+        index = field_to_index[context_fields]
         instances_list[index].append(instance)
+        summary_fields_lists[index].append(summary_fields)
         jackknifing_flags[index].append(is_jackknifing)
 
         # Potentially run jackknifing for the peers
         if metric.requires_jackknifing() and instance.summarizer_type == 'peer':
-            jk_fields_list = metric.jackknifer.get_jackknifing_fields_list(fields)
+            jk_fields_list = metric.jackknifer.get_jackknifing_fields_list(context_fields)
             if jk_fields_list:
                 for jk_fields in jk_fields_list:
                     if jk_fields not in field_to_index:
                         field_to_index[jk_fields] = len(field_to_index)
                         fields_list.append(jk_fields)
                         instances_list.append([])
+                        summary_fields_lists.append([])
                         jackknifing_flags.append([])
 
                     index = field_to_index[jk_fields]
                     instances_list[index].append(instance)
+                    summary_fields_lists[index].append(summary_fields)
                     jackknifing_flags[index].append(True)
 
+    # Construct the arguments that will be passed to the scoring method
+    summary_args = []
+    for name in metric.required_summary_fields:
+        summary_args.append([[summary_fields[name].to_input() for summary_fields in summary_fields_list] for summary_fields_list in summary_fields_lists])
+
+    context_args = []
+    for name in metric.required_context_fields:
+        context_args.append([fields[name].to_input() for fields in fields_list])
+
     # Score the summaries
-    summaries_lists = [[instance.summary.to_input() for instance in instances] for instances in instances_list]
-    args = [[fields[name].to_input() for fields in fields_list] for name in metric.required_fields]
-    results_lists = metric.score_multi_all(summaries_lists, *args)
+    results_lists = metric.score_multi_all(*summary_args, *context_args)
 
     # Used to aggregate the jk results
     jk_results = defaultdict(lambda: defaultdict(list))
@@ -105,51 +170,39 @@ def score_instances(instances: List[EvalInstance], metrics: List[Metric]) -> Dic
     return metrics_dicts
 
 
+def save_score_results(metrics_dicts: Dict[str, Dict[str, Metrics]], output_file: str, silent: bool) -> None:
+    with JsonlWriter(output_file) as out:
+        for instance_id in sorted(metrics_dicts.keys()):
+            for summarizer_id in sorted(metrics_dicts[instance_id].keys()):
+                out.write(metrics_dicts[instance_id][summarizer_id])
+
+
 class ScoreSubcommand(Subcommand):
     @overrides
     def add_subparser(self, parser: argparse._SubParsersAction):
         description = 'Score all of the inputs to evaluate a metric'
         self.parser = parser.add_parser('score', description=description, help=description)
-        self.parser.add_argument(
-            'config',
-            type=str,
-            help='The config file that specifies the dataset reader and metrics'
-        )
-        self.parser.add_argument(
-            'output_jsonl',
-            type=str,
-            help='The path to where the input-level metrics should be written'
-        )
-        self.parser.add_argument(
-            '--log-file',
-            type=str,
-            help='The file where the log should be written'
-        )
-        self.parser.add_argument(
-            '--silent',
-            action='store_true',
-            help='Controls whether the log should be written to stdout'
-        )
-        self.parser.add_argument(
-            '--overrides',
-            type=str,
-            help='A serialized json that will override the parameters passed in "config"'
-        )
+        add_score_arguments(self.parser, True)
         self.parser.set_defaults(func=self.run)
 
     @overrides
     def run(self, args):
         prepare_global_logging(file_path=args.log_file, silent=args.silent)
 
+        import_module_and_submodules('sacrerouge')
+        include_packages = args.include_packages or []
+        for package in include_packages:
+            import_module_and_submodules(package)
+
         params = Params.from_file(args.config, args.overrides)
         dataset_reader = DatasetReader.from_params(params.pop('dataset_reader'))
         metrics = _load_metrics(params)
 
-        instances = dataset_reader.read()
+        input_files = params.pop('input_files')
+        if isinstance(input_files, str):
+            input_files = [input_files]
+
+        instances = dataset_reader.read(*input_files)
         metrics_dicts = score_instances(instances, metrics)
 
-        # Save the results to the output file
-        with JsonlWriter(args.output_jsonl) as out:
-            for instance_id in sorted(metrics_dicts.keys()):
-                for summarizer_id in sorted(metrics_dicts[instance_id].keys()):
-                    out.write(metrics_dicts[instance_id][summarizer_id])
+        save_score_results(metrics_dicts, args.output_jsonl, args.silent)
